
;**************************************************************************
;*                  4-Digit 7-Segment LED Display driver                  *
;**************************************************************************
;**************************************************************************
;*          v 1.10 - 10.04.2016                                           *
;*                                                                        *
;**************************************************************************
;*        Hardw.: PIC16F1829 used                                         *
;*        OSC.......: Internal osc. used     POWER.....:   3,6V extern.   *
;**************************************************************************

 INCLUDE "P16F1829.INC"

_U001			     EQU     B'00100110000100'		;see Page 48 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet

	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002

;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

v_tmp_char	EQU	H'22'
v_loop		EQU	H'23'
v_loop2		EQU	H'24'

v_pos0		EQU	H'25'
v_pos1		EQU	H'26'
v_pos2		EQU	H'27'
v_pos3		EQU	H'28'

v_curr_p	EQU	H'29'
;--------------------------------------------------------------------------
;==========================================================================
		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.

		ORG	4
		goto	ir_main		;interrupt service routine
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;              ++  RESET :  main boot routine  ++
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
init_main	nop
		clrf	BSR
		clrf	INTCON			;disable interrupts

		banksel	OPTION_REG
		movlw	B'00111111'
		movwf	OPTION_REG		;see datasheet Page 176
						;bit 7 = 0 --> Week pull-ups are enabled
		movlw	H'03'
		movwf	BSR			;select BANK 3
		clrf	ANSELA
		clrf	ANSELB
		clrf	ANSELC

		banksel	TRISA
		movlw	B'11001100'		;RA0-1 and RA3-4 -- CC drivers
		movwf	TRISA

		movlw	B'00000000'		;7-segment dirvers
		movwf	TRISC

		movlw	B'11111111'
		movwf	TRISB

		movf	OSCCON,0
		andlw	B'11111111'
		iorlw	B'01111000'
		movwf	OSCCON			;set 16MHz internal clock (bit 6:3 = 1111) - Datasheet Page 68

		clrf	BSR			;select BANK 0

		clrf	PORTC
		clrf	PORTA
		bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		bsf	PORTA,D'004'
		bsf	PORTA,D'005'

		goto	main_prg
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Interrupt service routine
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
ir_main		clrf	BSR
;the GIE bit of INTCON is cleared in HW
;-----------------------------------------------------------------------------
		movf	v_curr_p,0
		call	ishow_digits
;-----------------------------------------------------------------------------
irs_fin_prep	incf	v_curr_p,1
		movf	v_curr_p,0
		xorlw	D'004'
		btfsc	STATUS,Z
		clrf	v_curr_p		;start new round
;-----------------------------------------------------------------------------
;this part finalize the interrupt service routine
irs_fin		clrf	TMR2
		bcf	PIR1,TMR2IF		;clear the source of the interrupt
		retfie				;this instruction sets the bit GIE of INTCON as well
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
ishow_digits	brw
		goto	ishow_digit0
		goto	ishow_digit1
		goto	ishow_digit2
		goto	ishow_digit3
;-----------------------------------------------------------------------------
ishow_digit0	call	disp_off
		movf	v_pos0,0
		call	wm_char_set
		movwf	PORTC
		bcf	PORTA,D'000'
		return
;-----------------------------------------------------------------------------
ishow_digit1	call	disp_off
		movf	v_pos1,0
		call	wm_char_set
		movwf	PORTC
		bcf	PORTA,D'001'
		return
;-----------------------------------------------------------------------------
ishow_digit2	call	disp_off
		movf	v_pos2,0
		call	wm_char_set
		movwf	PORTC
		bcf	PORTA,D'004'
		return
;-----------------------------------------------------------------------------
ishow_digit3	call	disp_off
		movf	v_pos3,0
		call	wm_char_set
		movwf	PORTC
		bcf	PORTA,D'005'
		return
;-----------------------------------------------------------------------------
disp_off	bsf	PORTA,D'000'
		bsf	PORTA,D'001'
		bsf	PORTA,D'004'
		bsf	PORTA,D'005'
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Loop
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
main_prg	clrf	BSR
		movlw	H'FF'
		movwf	PORTA
		clrf	PORTC
		clrf	v_curr_p
		call	timer2_init

		goto	Main_Appl
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
wm_char_set	brw
		retlw	B'00111111'
		retlw	B'00000110'
		retlw	B'01011011'
		retlw	B'01001111'
		retlw	B'01100110'
		retlw	B'01101101'
		retlw	B'01111101'
		retlw	B'00100111'
		retlw	B'01111111'
		retlw	B'01101111'
		retlw	B'01000000'		;minus
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c		MOVLW   D'090'
		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;                              -- TIMER2 and Interrupt Setup --
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
timer2_init	movlw	B'00000011'
;registers T2CON, TMR2 and PR2 resides in Bank 0 - no bank selection required at this place
		movwf	T2CON
		clrf	TMR2
		movlw	D'002'
		movwf	PR2			;overflow after 002 pulses
;-----------------------------------------------------------------------------------------
;Interrupt setup
		banksel	PIE1		
		clrf	PIE1
		clrf	PIE2
		bsf	PIE1,TMR2IE		;Enables the TMR2 interrupt
		clrf	BSR			;select BANK 0
		movlw	B'11000000'
		movwf	INTCON

;at this moment is everything prepared - but TIMER2 is not running

		bsf	T2CON,D'002'		;start TIMER2
		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;              This is the business logic of Main Application
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
Main_Appl	movlw	H'00'
		movwf	v_pos0
		movwf	v_pos1
		movwf	v_pos2
		movwf	v_pos3

demo_loop	nop
		call	dr2
		incf	v_pos3,1
		movf	v_pos3,0
		xorlw	H'0A'
		btfss	STATUS,Z
		goto	demo_loop

		clrf	v_pos3
		incf	v_pos2,1
		movf	v_pos2,0
		xorlw	H'0A'
		btfss	STATUS,Z
		goto	demo_loop

		clrf	v_pos2
		incf	v_pos1,1
		movf	v_pos1,0
		xorlw	H'0A'
		btfss	STATUS,Z
		goto	demo_loop

		clrf	v_pos1
		incf	v_pos0,1
		movf	v_pos0,0
		xorlw	H'0A'
		btfss	STATUS,Z
		goto	demo_loop

		clrf	v_pos0
		goto	demo_loop
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
		END